use axum::{
	extract::ws::{self,Message}, response, routing, Router
};
use open_tavern::api::{Request, RequestError, Response};

#[tokio::main]
async fn main() {
	let app = Router::new()
		.route("/", routing::get(root))
		.route("/socket.js", routing::get(socket))
		.route("/ws", routing::get(ws_handler))
		;

	let listener = tokio::net::TcpListener::bind("0.0.0.0:3001").await.unwrap();
	axum::serve(listener, app).await.unwrap();
}

async fn ws_handler(ws: ws::WebSocketUpgrade) -> impl axum::response::IntoResponse {
	ws.on_upgrade(handle_socket)
}

async fn handle_socket(mut socket: ws::WebSocket) {
	let mut logged_in = false;
	println!("Got a new socket");
	loop {
		if let Some(msg) = socket.recv().await {
			if let Ok(msg) = msg {
				let response: Message = match msg {
					Message::Text(t) =>  {
						let req = serde_json::from_str::<Request>(&t).unwrap_or_default();
						println!("Got message: {:?}", t);
						match req {
								Request::Error => Message::Text(serde_json::to_string(&Response::Error(RequestError::InvalidRequest)).unwrap()),
								Request::Login(r) => if !logged_in {
									if r.username == "rusty" {
										logged_in = true;
										Message::Text(serde_json::to_string(&Response::Login(open_tavern::api::login::LoginResult { success: true })).unwrap())
									}
									else {
										Message::Text(serde_json::to_string(&Response::Login(open_tavern::api::login::LoginResult { success: false })).unwrap())
									}
								} else {
									Message::Text(serde_json::to_string(&Response::Error(open_tavern::api::RequestError::AlreadyLoggedIn)).unwrap())
								},
							}
					}
					ws::Message::Binary(_) => todo!(),
					ws::Message::Ping(_) => todo!(),
					ws::Message::Pong(_) => todo!(),
					ws::Message::Close(_) => break,
				};
				socket.send(response).await.expect("failed sending to socket");
			}
		}
		else {
			break;
		}
	}
	println!("Done with so-cat");
}

async fn root() -> axum::response::Html<&'static str> {
	response::Html(include_str!("../assets/web/index.html"))
}

async fn socket() -> &'static str {
	include_str!("../assets/web/socket.js")
}

